/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：Java 信息安全性设计准则
 * 项目描述：用于检查Java源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.java.ast.parser.ArgumentListTreeImpl;
import org.sonar.java.model.expression.AssignmentExpressionTreeImpl;
import org.sonar.java.model.expression.IdentifierTreeImpl;
import org.sonar.java.model.expression.LiteralTreeImpl;
import org.sonar.java.model.expression.MemberSelectExpressionTreeImpl;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * TODO HttpInputDataChecker
 *
 * @author RenFengJiang
 * @date 2024/1/12
 */
@Rule(key = "HttpInputDataChecker")
public class HttpInputDataChecker extends IssuableSubscriptionVisitor {
    @Override
    public List<Tree.Kind> nodesToVisit() {
        /**
         * Tree.Kind.METHOD：方法节点
         * Tree.Kind.BLOCK：方法的代码块节点
         * Tree.Kind.METHOD_INVOCATION： 方法的调用节点
         */
        return Collections.singletonList(Tree.Kind.BLOCK);
    }
    @Override
    public void visitNode(Tree tree) {
        BlockTree blockTree = (BlockTree) tree;
        MethodBodyVisitor methodBodyVisitor = new MethodBodyVisitor(this);
        blockTree.accept(methodBodyVisitor);

    }

    static class MethodBodyVisitor extends BaseTreeVisitor{
        private final HttpInputDataChecker checker;
        List<String> list = new ArrayList<>();
        MethodBodyVisitor(HttpInputDataChecker checker){
            this.checker = checker;
        }

        @Override
        public void visitIfStatement(IfStatementTree tree) {
            ExpressionTree condition = tree.condition();
            if(condition instanceof AssignmentExpressionTreeImpl){
                AssignmentExpressionTreeImpl assignmentExpressionTree = (AssignmentExpressionTreeImpl) condition;
                list.add(assignmentExpressionTree.variable().toString());
            }
        }

        @Override
        public void visitMethodInvocation(MethodInvocationTree tree) {
            ExpressionTree expressionTree = tree.methodSelect();
            if(expressionTree instanceof MemberSelectExpressionTreeImpl){
                MemberSelectExpressionTreeImpl memberSelectExpressionTree = (MemberSelectExpressionTreeImpl) expressionTree;
                if("setHeader".equals(memberSelectExpressionTree.identifier().name()) || "addHeader".equals(memberSelectExpressionTree.identifier().name())){
                    Arguments arguments = tree.arguments();
                    if(arguments instanceof ArgumentListTreeImpl){
                        for (ExpressionTree argument : (ArgumentListTreeImpl) arguments) {
                            if(argument instanceof LiteralTreeImpl){
                                LiteralTreeImpl literalTree = (LiteralTreeImpl) argument;
                                if("STRING_LITERAL".equals(literalTree.kind().name())){
                                    checker.context.reportIssue(checker, literalTree, "HTTP输入数据验证");
                                    break;
                                }
                            }else if(argument instanceof IdentifierTreeImpl){
                                IdentifierTreeImpl identifierTree = (IdentifierTreeImpl) argument;
                                if(!list.contains(identifierTree.name())){
                                    checker.context.reportIssue(checker, identifierTree, "HTTP输入数据验证");
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }




    }
}
